unit LameCoder;
{ MP3 encoder component based on Lame encoder
  Using some code from BladeEnc DLL Header by Jack Kallestrup.

  Version 1.1}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Lame_dll;

type

  ELameError = class(Exception);

  TLameVersion = class(TPersistent)
  private
    FBeV      : TBeVersion;
  public
    constructor Create;
    procedure Assign(Source : TPersistent); override;
  published
    property DLLMajorVersion : byte read FBEV.byDLLMajorVersion write FBEV.byDLLMajorVersion;
    property DLLMinorVersion : byte read FBEV.byDLLMinorVersion write FBEV.byDLLMinorVersion;
    property MajorVersion    : byte read FBEV.byMajorVersion    write FBEV.byMajorVersion;
    property MinorVersion    : byte read FBEV.byMinorVersion    write FBEV.byMinorVersion;
  end;

type
  TLameMode = (STEREO, JSTEREO, DUALCHANNEL, MONO);

type
  TProgressEvent = procedure(Sender: TObject; PercentComplete : Word) of object;
  TProcessFileEvent = procedure(Sender: TObject; Num : Word; FileName : string) of object;

type
  TWaveHeader = record
       Marker1:        Array[0..3] of Char;
       BytesFollowing: LongInt;
       Marker2:        Array[0..3] of Char;
       Marker3:        Array[0..3] of Char;
       Fixed1:         LongInt;
       FormatTag:      Word;
       Channels:       Word;
       SampleRate:     LongInt;
       BytesPerSecond: LongInt;
       BytesPerSample: Word;
       BitsPerSample:  Word;
       Marker4:        Array[0..3] of Char;
       DataBytes:      LongInt;
  end;


type
  TLameCoder = class(TComponent)
private
    { Private declarations }
    FSampleRate    : DWORD;
    FReSampleRate  : DWORD;
    FMode	   : INTEGER;
    FBitrate       : DWORD;
    FMaxBitrate    : DWORD;
    FQuality       : MPEG_QUALITY;
    FMpegVersion   : DWORD;
    FPsyModel      : DWORD;
    FEmphasis      : DWORD;
    FPrivate	    : BOOL;
    FCRC	    : BOOL;
    FCopyright	    : BOOL;
    FOriginal       : BOOL;
    FWriteVBRHeader : BOOL;
    FEnableVBR	    : BOOL;
    FVBRQuality     : integer;
    FInputFiles     : TStrings;
    FOutputFiles    : TStrings;
    FLameVersion    : TLameVersion;
    FDefaultExt     : string;
    FFileName       : string;
    FBeConfig       : TBeConfig;
    FHBeStream      : THBeStream;
    FNumSamples     : DWORD;
    FBufSize        : DWORD;
    FInputBuf       : pointer;
    FOutputBuf      : pointer;
    FMP3File        : TFileStream;
    FWAVFile        : TFileStream;
    FOnBeginProcess : TNotifyEvent;
    FOnBeginFile    : TProcessFileEvent;
    FOnEndFile      : TProcessFileEvent;
    FOnEndProcess   : TNotifyEvent;
    FOnProgress     : TProgressEvent;
    FCancelProcess  : Boolean;
    FWave           : TWaveHeader;
    procedure SetSampleRate(Value : DWORD);
    procedure SetBitrate(Value : DWORD);
    procedure SetEnableVBR(Value : BOOL);
    function GetLameVersion : TLameVersion;
    procedure SetInputFiles(Value : TStrings);
    procedure SetOutputFiles(Value : TStrings);
    procedure SetMode(Value : TLameMode);
    function GetMode : TLameMode;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Assign(Source : TPersistent); override;
    procedure ProcessFiles;
    function PrepareCoder : TBEERR;
    function UnPrepareCoder(Buf : pointer; var dwWrite : DWORD) : TBEERR;
    procedure CloseCoder;
    procedure CancelProcess;
    function EncodeBuffer(InBuf : pointer; OutBuf : pointer; var OutP : DWORD) : TBEERR;
    property NumSamples     : DWORD read FNumSamples default 0;
    property BufSize        : DWORD read FBufSize default 0;
  published
    { Published declarations }
    property SampleRate     : DWORD read FSampleRate write SetSampleRate default 44100;
    property Bitrate        : DWORD read FBitrate write SetBitrate default 128;
    property MaxBitrate     : DWORD read FMaxBitrate write FMaxBitrate default 320;
    property Quality        : MPEG_QUALITY read FQuality write FQuality default NORMAL_QUALITY;
    property Private	    : BOOL read FPrivate write FPrivate default false;
    property CRC	    : BOOL read FCRC write FCRC default true;
    property Mode           : TLameMode read GetMode write SetMode default STEREO;
    property Copyright	    : BOOL read FCopyright write FCopyright default false;
    property Original       : BOOL read FOriginal write FOriginal default false;
    property WriteVBRHeader : BOOL read FWriteVBRHeader write FWriteVBRHeader default false;
    property EnableVBR	    : BOOL read FEnableVBR write SetEnableVBR default false;
    property VBRQuality     : integer read FVBRQuality write FVBRQuality default 4;
    property InputFiles     : TStrings read FInputFiles write SetInputFiles;
    property OutputFiles    : TStrings read FOutputFiles write SetOutputFiles;
    property LameVersion    : TLameVersion read GetLameVersion;
    property DefaultExt     : string read FDefaultExt write FDefaultExt;
    property Mp3FileName    : string read FFileName write FFileName;
    property OnBeginProcess : TNotifyEvent read FOnBeginProcess write FOnBeginProcess;
    property OnBeginFile    : TProcessFileEvent read FOnBeginFile    write FOnBeginFile;
    property OnEndFile      : TProcessFileEvent read FOnEndFile      write FOnEndFile;
    property OnEndProcess   : TNotifyEvent read FOnEndProcess   write FOnEndProcess;
    property OnProgress     : TProgressEvent read FOnProgress   write FOnProgress;
end;

type
   TLameStream = class(TFileStream)
   private
     FLameCoder : TLameCoder;
     FOutBuf    : pointer;
   protected
     procedure SetLameCoder(Value : TLameCoder);
   public
     constructor Create(const FileName: string; Mode: Word);
     destructor Destroy; override;
     property LameCoder : TLameCoder read FLameCoder write SetLameCoder;
     function Write(const Buffer; Count : Longint) : Longint; override;
     function Read(var Buffer; Count: Longint): Longint; override;
   end;

procedure Register;
{$R LameCoder.res}
implementation

var Err : TBeErr;
    FileLength,
    Done,
    dwWrite,
    toRead,
    isRead,
    toWrite,
    IsWritten   : DWORD;
    FWAVFileName,
    FMp3FileName : string;
    PrcntComplete : Word;


constructor TLameVersion.Create;
begin
     inherited Create;
     beVersion(FBEV);
end;

procedure TLameVersion.Assign(Source : TPersistent);
begin
     if Source is TLameVersion then begin
        FBEV.byDLLMajorVersion:=TLameVersion(Source).FBEV.byDLLMajorVersion;
        FBEV.byDLLMinorVersion:=TLameVersion(Source).FBEV.byDLLMinorVersion;
        FBEV.byMajorVersion:=TLameVersion(Source).FBEV.byMajorVersion;
        FBEV.byMinorVersion:=TLameVersion(Source).FBEV.byMinorVersion;
        exit;
     end;
     inherited Assign(Source);
end;

constructor TLameCoder.Create (AOwner : TComponent);
begin
     inherited Create(AOwner);
     FSampleRate:=44100;
     FBitRate:=128;
     FMaxBitRate:=320;
     FQuality:=NORMAL_QUALITY;
     FCRC:=true;
     FVBRQuality:=4;
     FInputFiles:=TStringList.Create;
     FOutputFiles:=TStringList.Create;
     FLameVersion:=TLameVersion.Create;
     FDefaultExt:='.mp3';
end;

destructor TLameCoder.Destroy;
begin
     FInputFiles.Free;
     FOutputFiles.Free;
     FLameVersion.Free;
     inherited Destroy;
end;

procedure TLameCoder.Assign(Source : TPersistent);
begin
     if Source is TLameCoder then begin
        FSampleRate     :=TLameCoder(Source).FSampleRate;
        FReSampleRate   :=TLameCoder(Source).FReSampleRate;
        FMode	        :=TLameCoder(Source).FMode;
        FBitrate        :=TLameCoder(Source).FBitrate;
        FMaxBitrate     :=TLameCoder(Source).FMaxBitrate;
        FQuality        :=TLameCoder(Source).FQuality;
        FMpegVersion    :=TLameCoder(Source).FMpegVersion;
        FPsyModel       :=TLameCoder(Source).FPsyModel;
        FEmphasis       :=TLameCoder(Source).FEmphasis;
        FPrivate        :=TLameCoder(Source).FPrivate;
        FCRC	        :=TLameCoder(Source).FCRC;
        FCopyright      :=TLameCoder(Source).FCopyright;
        FOriginal       :=TLameCoder(Source).FOriginal;
        FWriteVBRHeader :=TLameCoder(Source).FWriteVBRHeader;
        FEnableVBR	:=TLameCoder(Source).FEnableVBR;
        FVBRQuality     :=TLameCoder(Source).FVBRQuality;
        Exit;
      end;
      inherited Assign(Source);
end;

procedure TLameCoder.SetSampleRate(Value : DWORD);
begin
     FSampleRate:=Value;
end;

procedure TLameCoder.SetBitrate(Value : DWORD);
begin
     FBitrate:=Value;
end;

procedure TLameCoder.SetEnableVBR(Value : BOOL);
begin
     FEnableVBR:=Value;
     FWriteVBRHeader:=Value;
end;

function TLameCoder.GetLameVersion : TLameVersion;
begin
     Result:=FLameVersion;
end;

procedure TLameCoder.SetInputFiles(Value : TStrings);
var i : integer;
begin
     with FInputFiles do begin
          Assign(Value);
          FOutputFiles.Clear;
          for i:=0 to Count-1 do
              FOutputFiles.Add(ChangeFileExt(Strings[i],FDefaultExt));
     end;
end;

procedure TLameCoder.SetOutputFiles(Value : TStrings);
begin
     FOutputFiles.Assign(Value);
end;

procedure TLameCoder.SetMode(Value : TLameMode);
begin
     case Value of
          STEREO      : FMode:=BE_MP3_MODE_STEREO;
          JSTEREO     : FMode:=BE_MP3_MODE_JSTEREO;
          DUALCHANNEL : FMode:=BE_MP3_MODE_DUALCHANNEL;
          MONO        : FMode:=BE_MP3_MODE_MONO;
     end;
end;

function TLameCoder.GetMode : TLameMode;
begin
     case FMode of
          BE_MP3_MODE_STEREO      : Result:=STEREO;
          BE_MP3_MODE_JSTEREO     : Result:=JSTEREO;
          BE_MP3_MODE_DUALCHANNEL : Result:=DUALCHANNEL;
          BE_MP3_MODE_MONO        : Result:=MONO;
     end;
end;

function TLameCoder.PrepareCoder : TBEERR;
begin

     FBeConfig.Format.dwConfig:=BE_CONFIG_LAME;
     with FBeConfig.Format.LHV1 do begin
          dwStructVersion :=1;
          dwStructSize    :=SizeOf(FBeConfig);
          dwSampleRate    :=FSampleRate;
          dwReSampleRate  :=0;
          nMode	          :=FMode;
          dwBitrate       :=FBitrate;
          dwMaxBitrate    :=FMaxBitrate;
          nQuality        :=DWORD(FQuality);
          dwMpegVersion   :=MPEG1;
          dwPsyModel      :=0;
          dwEmphasis      :=0;
          bPrivate	  :=FPrivate;
          bCRC	          :=FCRC;
          bCopyright	  :=FCopyright;
          bOriginal       :=FOriginal;
          bWriteVBRHeader :=FWriteVBRHeader;
          bEnableVBR	  :=FEnableVBR;
          nVBRQuality     :=FVBRQuality;
     end;
     Err := BeInitStream(FBeConfig, FNumSamples, FBufSize, FHBeStream);
     if Err <> BE_ERR_SUCCESSFUL then begin
        Result:=Err;
        Exit;
     end;

     GetMem(FOutputBuf,FBufSize);
     Result:=Err;
end;

function TLameCoder.UnprepareCoder(Buf : pointer; var dwWrite : DWORD) : TBEERR;
begin
   Result := beDeinitStream(FHBeStream, Buf, dwWrite);
end;

procedure TLameCoder.CloseCoder;
begin
     FreeMem(FOutputBuf);
     beCloseStream(FHBeStream);
end;

procedure TLameCoder.ProcessFiles;

procedure CleanUp;
begin
     FreeMem(FInputBuf);
     FWAVFile.Free;
     FMp3File.Free;
end;

var
   FNum : integer;
   InputBufSize : Word;
begin
     FCancelProcess:=false;
     if Assigned(FOnBeginProcess) then FOnBeginProcess(Self);

     for FNum := 0 to FInputFiles.Count-1 do begin
     PrepareCoder;

     if Assigned(FOnBeginFile) then FOnBeginFile(Self,FNum,FInputFiles[FNum]);

     FWAVFileName:=FInputFiles[FNum];
     FMp3FileName:=FOutputFiles[FNum];

     FWAVFile := TFileStream.Create(FWAVFileName,fmOpenRead);
     FMp3File := TFileStream.Create(FMp3FileName,fmCreate or fmShareDenyNone);

     FWAVFile.Read(FWave,SizeOf(TWaveHeader));


     if FWave.Marker2='WAVE' then begin
        InputBufSize:=FNumSamples*FWave.BytesPerSample;
        FileLength := FWAVFile.Size-SizeOf(TWaveHeader);
     end
     else begin
        if FMode=BE_MP3_MODE_MONO then InputBufSize:=FNumSamples else
           InputBufSize:=FNumSamples*2;
        FileLength := FWAVFile.Size;
        FWAVFile.Seek(0,soFromBeginning);
     end;

     GetMem(FInputBuf,InputBufSize);


     Done := 0;

     while Done <> FileLength do begin
           if (Done + (InputBufSize) < FileLength) then
           toRead := InputBufSize
           else
           toRead := FileLength - Done;

     isRead := FWAVFile.Read(FInputBuf^,toRead);
     if isRead <> toRead then begin
        Cleanup;
        CloseCoder;
        raise ELameError.Create('Read Error');
        Exit;
     end;

     Err := beEncodeChunk(FHBeStream,(toRead div 2), FInputBuf, FOutputBuf, toWrite);
      if Err <> BE_ERR_SUCCESSFUL then begin
        CleanUp;
        CloseCoder;
        raise ELameError.Create('beEncodeChunk failed '+IntToStr(Err));
        Exit;
     end;

     IsWritten := FMp3File.Write(FOutputBuf^,toWrite);
     if toWrite <> IsWritten then begin
        Cleanup;
        CloseCoder;
        raise ELameError.Create('Write Error');
        Exit;
     end;

     Done := Done + toRead;
     PrcntComplete:=Trunc((Done / FileLength)*100);
     if Assigned(FOnProgress) then FOnProgress(Self,PrcntComplete);
     Application.ProcessMessages;

     if FCancelProcess then begin
        Cleanup;
        CloseCoder;
        Exit;
     end;
   end; {while}

   UnprepareCoder(FOutputBuf,dwWrite);

   IsWritten := FMp3File.Write(FOutputBuf^,dwWrite);

   if dwWrite <> IsWritten then begin
      CleanUp;
      CloseCoder;
      raise ELameError.Create('Write error');
      Exit;
   end;

   CleanUp;
   CloseCoder;
   if Assigned(FOnEndFile) then FOnEndFile(Self,FNum,FOutputFiles[FNum]);
  end; {for}

  if Assigned(FOnEndProcess) then FOnEndProcess(Self);

end;

procedure TLameCoder.CancelProcess;
begin
     FCancelProcess:=true;
end;

function TLameCoder.EncodeBuffer(InBuf :pointer; OutBuf : pointer; var OutP : DWORD) : TBEERR;
begin
     Result := beEncodeChunk(FHBeStream, FNumSamples, InBuf, OutBuf, OutP);
end;

constructor TLameStream.Create(const FileName: string; Mode: Word);
begin
     FLameCoder:=TLameCoder.Create(Application);
     inherited Create(FileName,fmCreate or fmShareDenyNone);
     FLameCoder.PrepareCoder;
     GetMem(FOutBuf,FLameCoder.BufSize);
end;

destructor TLameStream.Destroy;
var FCount : DWORD;
begin
     FLameCoder.UnprepareCoder(FOutBuf,FCount);
     inherited Write(FOutBuf^,FCount);
     FLameCoder.CloseCoder;
     FLameCoder.Free;
     FreeMem(FOutBuf);
     inherited Destroy;
end;

function TLameStream.Write(const Buffer; Count : Longint) : Longint;
var FCount : DWORD;
begin
     if FLameCoder.EncodeBuffer(pointer(Buffer),FOutBuf,FCount) = BE_ERR_SUCCESSFUL then
         Result:= inherited Write(FOutBuf^,FCount)
     else
         Result:=0;
end;

function TLameStream.Read(var Buffer; Count: Longint): Longint;
begin
     raise EStreamError.Create('Read from stream not supported');
end;

procedure TLameStream.SetLameCoder(Value : TLameCoder);
begin
     FLameCoder.CloseCoder;
     FLameCoder.Assign(Value);
     FLameCoder.PrepareCoder;
end;

procedure Register;
begin
  RegisterComponents('Lame', [TLameCoder]);
end;

end.
